# frozen_string_literal: true

require_relative "./type_checker_error"
require_relative "./command_runner"

module Rails
  module Tc
    class TypeChecker
      ERROR_LINE_REGEX = %r{^([^:]+):([0-9]+): (.*) https://srb\.help/([0-9]+)$}.freeze

      def self.set_up?
        File.exist?(Rails.root.join("sorbet/config"))
      end

      def initialize(ignored_errors = [], ignored_patterns = [])
        @ignored_errors = ignored_errors
        @ignored_patterns = ignored_patterns
      end

      def errors
        @errors ||= run
      end

      protected

      def run
        errors = []
        current = T.let(nil, T.nilable(TypeCheckerError))

        CommandRunner.type_check.split("\n", -1).each do |line|
          result = line.scan ERROR_LINE_REGEX

          if result.any?
            current = TypeCheckerError.new(
              result.first[0],
              result.first[1].to_i,
              result.first[3].to_i,
              result.first[2]
            )

            errors.push current
          elsif result.empty? && !current.nil? && line.start_with?("  ")
            current.notes.push line
          end
        end

        errors.reject do |error|
          @ignored_errors.include?(error.code) ||
            @ignored_patterns.any? { |pattern| File.fnmatch?(pattern, error.filename) }
        end
      end
    end
  end
end
